% TropD Calculate the Tropopause Height in isobaric coordinates 
% Written by Ori Adam Mar.17.2017
% based on the method described in Birner (2010), according to the WMO definition: 
% first level at which the lapse rate <= 2K/km and for which the lapse rate <= 2K/km in all levels at least 2km above the found level 
% Syntax:
% >> Pt = TropD_Calculate_TropopauseHeight(T,p)
% or
% >> [Pt , Ht] = TropD_Calculate_TropopauseHeight(T,p,Z)
% Input:
% T(lat,lev) or T(lon,lat,lev) = temperature (K). 
% P = pressure levels in hPa
% Z (optional) = geopotential height [m] or any field with the same dimensions as T
% Output:
% Pt(lat) or Pt(lon,lat) = tropopause level in hPa
% Ht(lat) or Ht(lon,lat) = the field Z evaluated at the tropopause. For Z=geopotential heigt, Ht is the tropopause altitude in m 

function [Pt , Ht]= TropD_Calculate_TropopauseHeight(T,P,Z)

Rd       = 287.04;      % Gas constant for dry air [J/kg/K]
Cpd      = 1005.7;      % Specific heat dry air [J/kg/K]
g        = 9.80616;     % Gravitational acceleration [m/s^2]

k = Rd/Cpd ;
PI = (fliplr(1:1000)*100).^k ;
Factor = g/Cpd*1000 ;
% Remove extra dimensions
if nargin >2
    Z = squeeze(Z) ; 
end
T = squeeze(T) ; 
if length(size(T)) == 2
    T = reshape(T,1,size(T,1),size(T,2)) ;
end
if nargin > 2 && length(size(Z)) == 2
    Z = reshape(Z,1,size(Z,1),size(Z,2)) ;
end
if P(end) >  P(1)
    [pm Ip] = max(size(P)) ;
    P = flipdim(P,Ip)    ;
    T = flipdim(T,3) ;
    if nargin > 2
       Z = flipdim(Z,3) ;
    end
end
Pk = repmat(reshape((P*100).^k,1,1,length(P)),[size(T,1) size(T,2) 1]) ;
Pk2 = (Pk(:,:,1:end-1) + Pk(:,:,2:end))/2 ;
%
T2  = (T(:,:,1:end-1) + T(:,:,2:end))/2 ;
Pk1 = squeeze(Pk2(1,1,:)) ;

Gamma = reshape((T(:,:,2:end)-T(:,:,1:end-1))./(Pk(:,:,2:end)-Pk(:,:,1:end-1)).*Pk2./T2.*Factor,size(T,1)*size(T,2),size(T,3)-1) ; % Lapse rate in K/km
T2    = reshape(T2,size(T,1)*size(T,2),size(T,3)-1) ;
%
Pt = zeros(size(T,1)*size(T,2),1) ;
for j = 1:size(Gamma,1)
    G1 = interp1(Pk1,double(Gamma(j,:)'),PI,'linear','extrap') ;
    T1 = interp1(Pk1,double(T2(j,:)'),PI,'linear','extrap') ;
    idx = find(G1 <=2 & PI < (550*100).^k & PI > (75*100).^k) ;
    Pidx = PI(idx) ;
    if isempty(Pidx)
        Pt(j) = nan ;
    else
        for c = 1:length(Pidx)
            dpk_2km = -2000*k*g/Rd./T1(c).*Pidx(c) ;
            idx2 = dsearchn(Pidx(c:end)',Pidx(c) + dpk_2km) ;
            if sum(G1(idx(c):idx(c)+idx2-1) <=2) == idx2  % Applying the condition that at no level in the above 2km the lapse rate >2K/km 
                Pt(j) = Pidx(c) ;
                break ;
            else
                continue ;
            end
        end
    end
end

Pt = Pt.^(1/k)/100 ;
if nargin > 2 
  if isequal(size(T),size(Z))
    Zt = reshape(Z,size(Z,1)*size(Z,2),size(Z,3)) ;
    Ht = zeros(size(T,1)*size(T,2),1) ;
    for j = 1:size(Ht,1)
    Ht(j) = interp1(P,Zt(j,:)',Pt(j)) ; 
    end
    Ht = reshape(Ht,size(T,1),size(T,2)) ;
  else
    disp('TropD_Calculate_TropopauseHeight: ERROR :  T and Z must have the same dimensions')
    Ht = [] ; 
  end
end
Pt = reshape(Pt,size(T,1),size(T,2)) ;

% QA
% figure ; 
% if length(size(T)) == 2
% plot(Pt) ; axis ij ;
% else
% contourf(Pt') ;  
% end

