% TropD EDJ metric
% Latitude of the eddy driven jet (EDJ) 
% Written by Ori Adam Mar.20.2017
% Methods:
% Latitude of maximum of the zonal wind at the level closest to the 850 hPa level

% Syntax:
% >> [PhiSH PhiNH] = TropD_Metric_EDJ(U,lat,lev,method)
% Input:
% U(lat,lev) = zonal mean zonal wind. Surface wind or uni-level zonal wind data can also be used as an input field using U(lat,1) 
% lat = equally spaced latitude column vector
% lev = vertical level vector in hPa units
% method (optional) = 'max' {default} |  'peak'
% Output:
% PhiSH = latitude of EDJ in the SH
% PhiNH = latitude of EDJ in the NH


function [PhiSH , PhiNH] = TropD_Metric_EDJ(U,lat,lev,method,n)

if nargin < 4
    method = 'max' ;
end
if nargin == 5
    if  ~isscalar(n) || n<1 
        disp('TropD_Metric_EDJ: ERROR : the smoothing parameter n must be >= 1' ) ;
        PhiNH = [] ;
        PhiSH = [] ;
        return
    end
else
    n = 0 ; % Flag for using default method values
end
eq_boundary    = 15 ;
polar_boundary = 60 ;
lev = double(lev) ;
try
    if ~iscolumn(lev)
        u = U(:,dsearchn(lev',850)) ;
    else
        u = U(:,dsearchn(lev,850)) ;
    end
catch % in case dsearchn is not supported
    u = U(:,find(lev==850)) ;
end
switch method
    case 'max'
        if n
        PhiNH = TropD_Calculate_MaxLat(u(lat>eq_boundary & lat < polar_boundary),lat(lat>eq_boundary & lat < polar_boundary),n) ;
        PhiSH = TropD_Calculate_MaxLat(u(lat>-polar_boundary & lat < -eq_boundary),lat(lat>-polar_boundary & lat < -eq_boundary),n) ;    
        else
        PhiNH = TropD_Calculate_MaxLat(u(lat>eq_boundary & lat < polar_boundary),lat(lat>eq_boundary & lat < polar_boundary)) ;
        PhiSH = TropD_Calculate_MaxLat(u(lat>-polar_boundary & lat < -eq_boundary),lat(lat>-polar_boundary & lat < -eq_boundary)) ;
        end
    case 'peak'
        if n
        PhiNH = TropD_Calculate_MaxLat(u(lat>eq_boundary & lat < polar_boundary),lat(lat>eq_boundary & lat < polar_boundary),n) ; % 
        PhiSH = TropD_Calculate_MaxLat(u(lat>-polar_boundary & lat < -eq_boundary),lat(lat>-polar_boundary & lat < -eq_boundary),n) ;    
        else
        PhiNH = TropD_Calculate_MaxLat(u(lat>eq_boundary & lat < polar_boundary),lat(lat>eq_boundary & lat < polar_boundary),30) ; % for n>=30, result converges to max(F)
        PhiSH = TropD_Calculate_MaxLat(u(lat>-polar_boundary & lat < -eq_boundary),lat(lat>-polar_boundary & lat < -eq_boundary),30) ;    
        end
    otherwise
        disp(['TropD_Metric_EDJ: ERROR: unrecognized method ''',method,''''])
        PhiNH = [] ;
        PhiSH = [] ;
end

% QA
% figure ;
% plot(lat,u,'-k','linewidth',2) ; hold on
% plot([PhiNH  PhiNH],[min(u)  max(u)],'--')
% plot([PhiSH  PhiSH],[min(u)  max(u)],'--')
% xlabel('Latitude','fontsize',14) ;
% ylabel('850hPa zonal wind','fontsize',14)
% xlim([-90 90])