% TropD Outgoing Longwave Radiation (OLR) metric
% Written by Ori Adam Mar.21.2017
% Methods:
%  '250W'{Default}: the first latitude poleward of the tropical OLR maximum in each hemisphere where OLR crosses 250W/m^2
%  '20W': the first latitude poleward of the tropical OLR maximum in each hemisphere where OLR crosses the tropical OLR max minus 20W/m^2
%  'cutoff': the first latitude poleward of the tropical OLR maximum in each hemisphere where OLR crosses a specified cutoff value
%  '10Perc': the first latitude poleward of the tropical OLR maximum in each hemisphere where OLR is 10% smaller than the tropical OLR maximum
%  'max': the latitude of maximum of tropical olr in each hemisphere with the smoothing paramerer n=6 in TropD_Calculate_MaxLat
%  'peak': the latitude of maximum of tropical olr in each hemisphere with the smoothing parameter n=30 in TropD_Calculate_MaxLat
% Syntax:
% >> [PhiSH PhiNH] = TropD_Metric_OLR(olr,lat,method,Cutoff)
% Input:
% olr(lat) = zonal mean TOA olr (assumed positive)
% lat = equally spaced latitude column vector
% method (optional) = '250W' {default} | 'cutoff' | '10Perc' | '20W' | 'max' | 'peak'
% Cutoff = scalar (optional). For the method 'cutoff', Cutoff specifies the OLR cutoff value. For the 'max' method, Cutoff (optional, default=6) specifies the smoothing parameter n in TropD_Calculate_MaxLat
% Output:
% PhiSH = latitude of near equator OLR threshold crossing in the SH
% PhiNH = latitude of near equator OLR threshold crossing in the NH

function [PhiSH , PhiNH] = TropD_Metric_OLR(olr,lat,method,Cutoff)

if nargin < 3
    method = '250W' ;
end
% make latitude vector monotonically increasing
if lat(end) < lat(1)
    olr = flip(olr) ;
    lat = flip(lat)   ;
end
if ~iscolumn(lat)
    lat = lat'   ;
end
if ~iscolumn(olr)
    olr = olr'   ;
end
eq_boundary       = 5  ;
subpolar_boundary = 40 ;
polar_boundary    = 60 ;
% NH
olr_max_lat_NH = TropD_Calculate_MaxLat(olr(lat>eq_boundary & lat < subpolar_boundary),lat(lat>eq_boundary & lat < subpolar_boundary)) ;
olr_max_NH     = max(olr(lat>eq_boundary & lat < subpolar_boundary)) ;
% SH
olr_max_lat_SH = TropD_Calculate_MaxLat(olr(lat>-subpolar_boundary & lat < -eq_boundary),lat(lat>-subpolar_boundary & lat < -eq_boundary)) ;
olr_max_SH     = max(olr(lat>-subpolar_boundary & lat < -eq_boundary)) ;
switch method
    case '20W'
        PhiNH          = TropD_Calculate_ZeroCrossing(olr(lat>olr_max_lat_NH & lat < polar_boundary) - olr_max_NH + 20,lat(lat>olr_max_lat_NH & lat < polar_boundary)) ;
        PhiSH          = TropD_Calculate_ZeroCrossing(flip(olr(lat<olr_max_lat_SH & lat > -polar_boundary)) - olr_max_SH + 20,flip(lat(lat<olr_max_lat_SH & lat > -polar_boundary))) ; % flip in order to find the most equatorward zero crossing
    case '250W'
        PhiNH          = TropD_Calculate_ZeroCrossing(olr(lat>olr_max_lat_NH & lat < polar_boundary) - 250,lat(lat>olr_max_lat_NH & lat < polar_boundary)) ;
        PhiSH          = TropD_Calculate_ZeroCrossing(flip(olr(lat<olr_max_lat_SH & lat > -polar_boundary)) - 250,flip(lat(lat<olr_max_lat_SH & lat > -polar_boundary))) ; % flip in order to find the most equatorward zero crossing
    case 'cutoff'
        if nargin < 4
            Cutoff = 250 ;
        end
        PhiNH          = TropD_Calculate_ZeroCrossing(olr(lat>olr_max_lat_NH & lat < polar_boundary) - Cutoff,lat(lat>olr_max_lat_NH & lat < polar_boundary)) ;
        PhiSH          = TropD_Calculate_ZeroCrossing(flip(olr(lat<olr_max_lat_SH & lat > -polar_boundary)) - Cutoff,flip(lat(lat<olr_max_lat_SH & lat > -polar_boundary))) ; % flip in order to find the most equatorward zero crossing
    case '10Perc'
        PhiNH          = TropD_Calculate_ZeroCrossing(olr(lat>olr_max_lat_NH & lat < polar_boundary)/olr_max_NH -0.9,lat(lat>olr_max_lat_NH & lat < polar_boundary)) ;
        PhiSH          = TropD_Calculate_ZeroCrossing(flip(olr(lat<olr_max_lat_SH & lat > -polar_boundary))/olr_max_SH -0.9,flip(lat(lat<olr_max_lat_SH & lat > -polar_boundary))) ; % flip in order to find the most equatorward zero crossing
    case 'max'
        if nargin == 4
            if  isscalar(Cutoff) && Cutoff>=1
                % NH
                PhiNH = TropD_Calculate_MaxLat(olr(lat>eq_boundary & lat < subpolar_boundary),lat(lat>eq_boundary & lat < subpolar_boundary),Cutoff) ;
                % SH
                PhiSH = TropD_Calculate_MaxLat(olr(lat>-subpolar_boundary & lat < -eq_boundary),lat(lat>-subpolar_boundary & lat < -eq_boundary),Cutoff) ;
            else
                disp(['TropD_Metric_OLR: ERROR : the smoothing paramter n must be >= 1']) ;
                PhiNH = [] ;
                PhiSH = [] ;
            end
        else
            PhiNH = olr_max_lat_NH ;
            PhiSH = olr_max_lat_SH ;
        end
    case 'peak'
        PhiNH = TropD_Calculate_MaxLat(olr(lat>eq_boundary & lat < subpolar_boundary),lat(lat>eq_boundary & lat < subpolar_boundary),30) ; % for n>=30, result converges to max(F)
        PhiSH = TropD_Calculate_MaxLat(olr(lat>-subpolar_boundary & lat < -eq_boundary),lat(lat>-subpolar_boundary & lat < -eq_boundary),30) ;
    otherwise
        disp(['TropD_Metric_OLR: unrecognized method ''',method,''''])
        PhiNH = [] ;
        PhiSH = [] ;
        return
end


% QA
% figure ;
% plot(lat,olr,'-k','linewidth',2) ; hold on
% plot([PhiNH  PhiNH],[max(olr) - 40  max(olr) + 10],'--k')
% plot([PhiSH  PhiSH],[max(olr) - 40  max(olr) + 10],'--k')
% xlabel('Latitude','fontsize',14) ;
% ylabel('OLR','fontsize',14)
