% TropD Precipitation minus Evaporation (PE) metric
% Written by Ori Adam Mar.21.2017
% Methods:
%  'zero_crossing': the first latitude poleward of the subtropical minimum where P-E changes from negative to positive values
% Syntax:
% >> [PhiSH PhiNH] = TropD_Metric_PE(pe,lat,method,Lat_Uncertainty)
% Input:
% pe(lat) = zonal-mean precipitation minus evaporation
% lat = equally spaced latitude column vector
% method (optional) = 'zero_crossing' {default}
% Lat_Uncertainty (optional) = [Degrees latitude, default = 0] the minimal distance allowed between the first and second zero crossings along lat
% Output:
% PhiSH = latitude of first subtropical P-E zero crossing in the SH
% PhiNH = latitude of first subtropical P-E zero crossing in the NH

function [PhiSH , PhiNH] = TropD_Metric_PE(pe,lat,method,Lat_Uncertainty)

if nargin < 3
    method = 'zero_crossing' ;
end
if nargin < 4
    Lat_Uncertainty = 0 ;
end
% make latitude vector monotonically increasing
if lat(end) < lat(1)
    pe = flip(pe) ;
    lat = flip(lat)   ;
end
if ~iscolumn(lat)
    lat = lat'   ;
end
if ~iscolumn(pe)
    pe = pe'   ;
end
% The gradient of PE is used to determine whether PE becomes positive at the zero crossing
ped = interp1((lat(1:end-1)+lat(2:end))/2,diff(pe),lat) ;

eq_boundary       = 5  ;
subpolar_boundary = 50 ;
polar_boundary    = 70 ;
switch method
    case 'zero_crossing'
        % NH
        M1        = TropD_Calculate_MaxLat(-pe(lat>eq_boundary & lat < subpolar_boundary),lat(lat>eq_boundary & lat < subpolar_boundary),30) ;
        ZC1       = TropD_Calculate_ZeroCrossing(pe(lat>M1 & lat < polar_boundary),lat(lat>M1 & lat < polar_boundary),Lat_Uncertainty) ;
        if interp1(lat,ped,ZC1) > 0
            PhiNH = ZC1 ;
        else
            PhiNH     = TropD_Calculate_ZeroCrossing(pe(lat>ZC1 & lat < polar_boundary),lat(lat>ZC1 & lat < polar_boundary),Lat_Uncertainty) ;
        end
        % SH
        M1        = TropD_Calculate_MaxLat(flip(-pe(lat<-eq_boundary & lat > -subpolar_boundary)),flip(lat(lat<-eq_boundary & lat > -subpolar_boundary)),30) ; % flip in order to find the most equatorward zero crossing
        ZC1       = TropD_Calculate_ZeroCrossing(flip(pe(lat<M1 & lat > -polar_boundary)),flip(lat(lat<M1 & lat > -polar_boundary)),Lat_Uncertainty) ; % flip in order to find the most equatorward zero crossing
        if interp1(lat,ped,ZC1) < 0
            PhiSH = ZC1 ;
        else
            PhiSH     = TropD_Calculate_ZeroCrossing(flip(pe(lat<ZC1 & lat > -polar_boundary)),flip(lat(lat<ZC1 & lat > -polar_boundary)),Lat_Uncertainty) ; % flip in order to find the most equatorward zero crossing
        end
    otherwise
        disp(['TropD_Metric_PE: ERROR : unrecognized method ''',method,''''])
        PhiNH = [] ;
        PhiSH = [] ;
        return
        
end
% QA
% figure ;
% plot(lat,pe*0,'-k','linewidth',1) ; hold on
% plot(lat,pe,'-k','linewidth',2) ;
% plot([PhiNH  PhiNH],[max(pe)  min(pe)],'--k')
% plot([PhiSH  PhiSH],[min(pe)  max(pe)],'--k')
% xlabel('Latitude','fontsize',14) ;
% ylabel('P minus E','fontsize',14)
% 
