% TropD Tropopause break (TPB) metric
% Written by Ori Adam Mar.17.2017
% Methods:
% 'max_gradient': The latitude of maximal poleward gradient of the tropopause height
% 'cutoff': The most equatorward latitude where the tropopause crosses a prescribed cutoff value
% 'max_potemp': The latitude of maximal difference between the potential temperature at the tropopause and at the surface

% Syntax:
% >> [PhiSH PhiNH] = TropD_Metric_TPB(T,lat,lev,method)
% Or
% >> [PhiSH PhiNH] = TropD_Metric_TPB(T,lat,lev,method,Z)
% Or
% >> [PhiSH PhiNH] = TropD_Metric_TPB(T,lat,lev,method,Z,Cutoff)
% Input:
% T(lat,lev) = temperature (K)
% lat = equally spaced latitude column vector
% lev = pressure levels column vector in hPa
% method (optional) = 'max_gradient' {default} |  'max_potemp'  |  'cutoff'  |
% Z(lat,lev) (optional) = geopotential height (m)
% Cutoff (optional, scalar) = geopotential height (m) cutoff that marks the location of the tropopause break; Default value = 15000m
% Output:
% PhiSH = latitude of tropopause break in the SH
% PhiNH = latitude of tropopause break in the NH

function [PhiSH , PhiNH] = TropD_Metric_TPB(T,lat,lev,method,Z,Cutoff)

if nargin < 4
    method = 'max_gradient' ;
end

polar_boundary = 70 ;

switch method
    case 'max_gradient'
        Pt = TropD_Calculate_TropopauseHeight(T,lev) ;
        Ptd = diff(Pt)/(lat(2)-lat(1)) ;
        lat2 = ((lat(2:end) + lat(1:end-1))/2)';
        if nargin == 5
            if isscalar(Z) && Z>=1
                PhiNH = TropD_Calculate_MaxLat(Ptd(lat2>0 & lat2 < polar_boundary),lat2(lat2>0 & lat2 < polar_boundary),Z) ;  % n = Z
                PhiSH = TropD_Calculate_MaxLat(-Ptd(lat2>-polar_boundary & lat2 < 0),lat2(lat2>-polar_boundary & lat2 < 0),Z) ; % n = Z
            else
                disp('TropD_Metric_TPB: ERROR : the smoothing parameter n must be >= 1' ) ;
                PhiNH = [] ;
                PhiSH = [] ;
                return
            end
        else
            PhiNH = TropD_Calculate_MaxLat(Ptd(lat2>0 & lat2 < polar_boundary),lat2(lat2>0 & lat2 < polar_boundary)) ; % Default value n = 6
            PhiSH = TropD_Calculate_MaxLat(-Ptd(lat2>-polar_boundary & lat2 < 0),lat2(lat2>-polar_boundary & lat2 < 0)) ; % Default value n = 6
        end
    case 'max_potemp'
        Rd       = 287.04;      % Gas constant for dry air [J/kg/K]
        Cpd      = 1005.7;      % Specific heat dry air [J/kg/K]
        k = Rd/Cpd ;
        if iscolumn(lev)
            XF = repmat((lev'/1000).^(k),[length(lat) 1]); % Exner function
        else
            XF = repmat((lev/1000).^(k),[length(lat) 1]);
        end
        PT = T./XF ;
        [Pt , PTt] = TropD_Calculate_TropopauseHeight(T,lev,PT) ;
        try
            PTdif = PTt' - nanmin(PT,[],2) ;  % use nanmean to avoid subsurface data
        catch % nanmean not supported in some MATLAB licenses
            PTdif = PTt' - min(PT,[],2) ; % hopefully, no nan values ...
        end
        if nargin == 5
            if isscalar(Z) && Z>=1
                PhiNH = TropD_Calculate_MaxLat(PTdif(lat>0 & lat < polar_boundary) ,lat(lat>0 & lat < polar_boundary ),Z) ;  % n = Z
                PhiSH = TropD_Calculate_MaxLat(PTdif(lat>-polar_boundary & lat < 0),lat(lat>-polar_boundary & lat < 0),Z) ;  % n = Z
            else
                disp('TropD_Metric_TPB: ERROR : the smoothing parameter n must be >= 1' ) ;
                PhiNH = [] ;
                PhiSH = [] ;
                return
            end
        else % Default value n = 6
            PhiNH = TropD_Calculate_MaxLat(PTdif(lat>0 & lat < polar_boundary) ,lat(lat>0 & lat < polar_boundary)) ;
            PhiSH = TropD_Calculate_MaxLat(PTdif(lat>-polar_boundary & lat < 0),lat(lat>-polar_boundary & lat < 0)) ;
        end
        
    case 'cutoff'
        if nargin < 6
            Cutoff   = 15*1000 ;
        end
        [Pt , Ht] = TropD_Calculate_TropopauseHeight(T,lev,Z) ;
        if lat(end) < lat(1)
            Ht = flip(Ht) ;
            lat = flip(lat)   ;
        end
        if ~iscolumn(lat)
            lat = lat'   ;
        end
        if ~iscolumn(Ht)
            Ht = Ht'   ;
        end
        polar_boundary = 60      ;
        PhiNH = TropD_Calculate_ZeroCrossing(Ht(lat>0 & lat < polar_boundary) - Cutoff,lat(lat>0 & lat < polar_boundary)) ;
        % flip in order to find the most equatorward zero crossing
        PhiSH = TropD_Calculate_ZeroCrossing(flip(Ht(lat<0 & lat > -polar_boundary)) - Cutoff,flip(lat(lat<0 & lat > -polar_boundary))) ;
    otherwise
        disp(['TropD_Metric_TPB: ERROR: unrecognized method ''',method,''''])
        PhiNH = [] ;
        PhiSH = [] ;
        return
end

% QA
% Fig = figure ;
% plot(lat,Pt,'-k','linewidth',2) ; hold on
% switch method
%     case 'max_gradient'
%         Color = [0 1 0] ;
%     case 'max_potemp'
%         Color = [0 0 1] ;
%         plot(lat,PTdif,'-m')
%     case 'cutoff_height'
%         Color = [1 0 0] ;
%     otherwise
%         Color = [0 0 0] ;
% end
% plot([PhiNH  PhiNH],[min(Pt)  max(Pt)],'--','color',Color)
% plot([PhiSH  PhiSH],[min(Pt)  max(Pt)],'--','color',Color)
% axis ij
% xlabel('Latitude','fontsize',14) ;
% ylabel('Tropopause level','fontsize',14)
% pause ;
% close(Fig) ;
