% TropD near-surface zonal wind metric
% Written by Ori Adam Mar.21.2017
% Methods:
%  'zero_crossing': the first subtropical latitude where near-surface zonal wind changes from negative to positive
% Syntax:
% >> [PhiSH PhiNH] = TropD_Metric_UAS(U,lat,lev)
% or
% >> [PhiSH PhiNH] = TropD_Metric_UAS(U,lat,lev,method,Lat_Uncertainty)
% Input:
% U(lat,lev) = zonal mean zonal wind
% or 
% U(lat,1) = zonal mean surface wind
% lat = equally spaced latitude column vector
% lev = vertical level vector in hPa units 
% (set lev=1 for single-level input zonal wind U(lat,1))
% method (optional) = 'zero_crossing' {default}
% Lat_Uncertainty (optional) = [Degrees latitude, default = 0] the minimal distance allowed between the first and second zero crossings
% Output:
% PhiSH = latitude of first subtropical zero crossing in the SH
% PhiNH = latitude of first subtropical zero crossing in the NH


function [PhiSH , PhiNH] = TropD_Metric_UAS(U,lat,lev,method,Lat_Uncertainty)

if nargin < 3
    lev = 1 ; 
end
if nargin < 4
    method = 'zero_crossing' ;
end
if nargin < 5
    Lat_Uncertainty = 0 ;
end
% uas = single-level zonal wind
lev = double(lev) ;
if length(lev) > 1
    try
        if ~iscolumn(lev)
            uas = U(:,dsearchn(lev',850)) ;
        else
            uas = U(:,dsearchn(lev,850)) ;
        end
    catch % in case dsearchn is not supported
        uas = U(:,find(lev==850)) ;
    end
else
    uas = U ;
end
% make latitude vector monotonically increasing
if lat(end) < lat(1)
    uas = flip(uas) ;
    lat = flip(lat)   ;
end
if ~iscolumn(lat)
    lat = lat'   ;
end
if ~iscolumn(uas)
    uas = uas'   ;
end
eq_boundary       = 5  ;
subpolar_boundary = 30 ;
polar_boundary    = 60 ;
% NH
uas_min_lat_NH = TropD_Calculate_MaxLat(-uas(lat>eq_boundary & lat < subpolar_boundary),lat(lat>eq_boundary & lat < subpolar_boundary)) ;
% SH
uas_min_lat_SH = TropD_Calculate_MaxLat(-uas(lat>-subpolar_boundary & lat < -eq_boundary),lat(lat>-subpolar_boundary & lat < -eq_boundary)) ;
switch method
    case 'zero_crossing'
        PhiNH          = TropD_Calculate_ZeroCrossing(uas(lat>uas_min_lat_NH & lat < polar_boundary),lat(lat>uas_min_lat_NH & lat < polar_boundary),Lat_Uncertainty) ;
        PhiSH          = TropD_Calculate_ZeroCrossing(flip(uas(lat<uas_min_lat_SH & lat > -polar_boundary)),flip(lat(lat<uas_min_lat_SH & lat > -polar_boundary)),Lat_Uncertainty) ; % flip in order to find the most equatorward zero crossing
    otherwise
        disp(['TropD_Metric_UAS: ERROR : unrecognized method ''',method,''''])
        PhiNH = [] ;
        PhiSH = [] ;
        return
end

% QA
%  figure ;
%  plot(lat,uas*0,'-k','linewidth',1) ; hold on
%  plot(lat,uas,'-k','linewidth',2) ;
%  plot([PhiNH  PhiNH],[max(uas)  min(uas)],'--k')
%  plot([PhiSH  PhiSH],[min(uas)  max(uas)],'--k')
%  xlabel('Latitude','fontsize',14) ;
%  ylabel('Near-surface zonal wind','fontsize',14)

